#include "anircd.h"

void cmd_register(User *u)
{
	u->flags |= UFL_REG;

	/* XXX: configurable 'network' variable */
	send_num(u, 1, NULL, ":Welcome to the HTS-Anarchy IRC Network %s!%s@%s", u->nick, u->ident, u->host);
	send_num(u, 2, NULL, ":Your host is %s, running version AnarchyIRCd", c_sname);
	send_num(u, 3, NULL, ":This server was created Wed Jul 5 2006 at 00:00:00 UTC");
	send_num(u, 4, NULL, "%s AnarchyIRCd", c_sname);

	cmd_do_lusers(u);

	/* XXX: implement proper MOTD support */
	send_num(u, 375, NULL, ":- %s Message of the Day -", c_sname);
	send_num(u, 372, NULL, ":- This is a minimalistic IRC network that engenders powerlessness and anonymity.");
	send_num(u, 376, NULL, ":End of /MOTD command");
}

void cmd_user(User *u, char *cmd, int ac, char **av)
{
	if (ac < 4)
	{
		send_num(u, 461, cmd, ":Not enough parameters");
		return;
	}

	u->host = strdup("no.host");
	u->realname = strdup(av[3]);
	u->ident = strdup(av[0]);

	u->flags |= UFL_HAVEUSER;
}

void cmd_nick(User *u, char *cmd, int ac, char **av)
{
	if (ac < 1)
	{
		send_num(u, 431, "", ":No nickname given");
		return;
	}

	if (!validate(av[0]) || strlen(av[0]) > MAX_NICKLEN)
		send_num(u, 432, av[0], ":Erroneous nickname");
	else if (user_find(av[0]))
		send_num(u, 433, av[0], ":Nickname is already in use");
	else
	{
		if (u->flags & UFL_REG)
		{
			send_ucmd_chanbutone(u, "NICK :%s", av[0]);
			send_ucmd_one(u, u, "NICK :%s", av[0]);
		}
		if (u->nick)
			free(u->nick);
		u->nick = strdup(av[0]);
		if (!(u->flags & UFL_HAVENICK))
			u->flags |= UFL_HAVENICK;
	}
}

void cmd_quit(User *u, char *cmd, int ac, char **av)
{
	char	reason[MAX_QUIT];

	snprintf(reason, sizeof(reason), "Quit: %s", ac >= 1 ? av[0] : "");
	user_kill(u, reason);
}

void cmd_names(User *u, char *cmd, int ac, char **av)
{
	if (ac < 1)
		send_num(u, 366, "*", ":End of /NAMES list");
	else
	{
		Chan	*c;

		c = chan_find(av[0]);
		if (!c)
			send_num(u, 403, av[0], ":No such channel");
		else
			cmd_do_names(u, c);
	}
}

void cmd_do_names(User *u, Chan *c)
{
	int	i;
	char	nbuf[300];

	*nbuf = '\0';
	for (i = 0; i < c->nusers; ++i)
	{
		strncat(nbuf, c->users[i]->nick, sizeof(nbuf) - strlen(nbuf) - 1);
		strncat(nbuf, " ", sizeof(nbuf) - strlen(nbuf) - 1);
		if (strlen(nbuf) >= 256)
		{
			send_num(u, 353, "=", "%s :%s", c->name, nbuf);
			*nbuf = '\0';
		}
	}
	if (*nbuf)
		send_num(u, 353, "=", "%s :%s", c->name, nbuf);

	send_num(u, 366, c->name, ":End of /NAMES list");
}

void cmd_join(User *u, char *cmd, int ac, char **av)
{
	char	**chav, *s, *t;
	int	chac, i;
	Chan	*c;

	if (ac < 1)
	{
		send_num(u, 461, cmd, ":Not enough parameters");
		return;
	}

	for (chac = 0, chav = NULL, s = av[0]; ; s = t)
	{
		chav = (char **) realloc(chav, sizeof(char *) * ++chac);
		chav[chac - 1] = s;
		if ((t = strchr(s, ',')))
			*t++ = (char) 0;
		else
			break;
	}

	for (i = 0; i < chac; ++i)
		if (!strcasecmp(chav[i], "0"))
		{
			int	j;

			for (j = 0; u->nchans; ++j)
			{
				send_ucmd_schan(u, u->chans[j], "PART :%s", u->chans[0]->name);
				chan_deluent(u, u->chans[0]);
			}
		}
		else if (!IsChan(chav[i]))
			send_num(u, 403, chav[i], ":No such channel");
		else
		{
			c = chan_find(chav[i]);
			if (!c)
				c = chan_new(chav[i]);
			/* Already on channel */
			if (user_findcidx(u, c) >= 0)
				continue;

			chan_adduent(u, c);
			send_ucmd_schan(u, c, "JOIN :%s", c->name);
			cmd_do_names(u, c);
			send_num(u, 324, c->name, "+");
		}

	free(chav);
}

void cmd_part(User *u, char *cmd, int ac, char **av)
{
	char	**chav, *s, *t;
	int	chac, i;
	Chan	*c;

	if (ac < 1)
	{
		send_num(u, 461, cmd, ":Not enough parameters");
		return;
	}

	for (chac = 0, chav = NULL, s = av[0]; ; s = t)
	{
		chav = (char **) realloc(chav, sizeof(char *) * ++chac);
		chav[chac - 1] = s;
		if ((t = strchr(s, ',')))
			*t++ = (char) 0;
		else
			break;
	}

	for (i = 0; i < chac; ++i)
		if (*chav[i] != '#')
			send_num(u, 403, chav[i], ":No such channel");
		else
		{
			c = chan_find(chav[i]);
			if (!c)
				send_num(u, 403, chav[i], ":No such channel");
			else if (user_findcidx(u, c) < 0)
				send_num(u, 442, chav[i], ":You're not on that channel");
			else
			{
				send_ucmd_schan(u, c, "PART :%s", c->name);
				chan_deluent(u, c);
			}
		}

	free(chav);
}

void cmd_gen_msg(User *u, char *cmd, int ac, char **av)
{
	char	**tav, *s, *t;
	int	tac, i;

	if (ac < 1)
		send_num(u, 411, NULL, ":No recipient given (%s)", cmd);
	else if (ac < 2 || !*av[1])
		send_num(u, 412, NULL, ":No text to send");
	else
	{
		for (tac = 0, tav = NULL, s = av[0]; ; s = t)
		{
			tav = (char **) realloc(tav, sizeof(char *) * ++tac);
			tav[tac - 1] = s;
			if ((t = strchr(s, ',')))
				*t++ = (char) 0;
			else
				break;
		}

		for (i = 0; i < tac; ++i)
			if (*tav[i] == '#')
			{
				Chan	*c;

				c = chan_find(tav[i]);
				if (!c)
				{
					send_num(u, 401, tav[i], ":No such nick/channel");
					continue;
				}
				if (user_findcidx(u, c) < 0)
				{
					send_num(u, 442, tav[i], ":You're not on that channel");
					continue;
				}

				send_ucmd_schanbutone(u, c, "%s %s :%s", cmd, tav[i], av[1]);
			}
			else
			{
				User *ut;

				ut = user_find(tav[i]);
				if (!ut)
				{
					send_num(u, 401, tav[i], ":No such nick/channel");
					continue;
				}
				send_ucmd_one(u, ut, "%s %s :%s", cmd, tav[i], av[1]);
			}

		free(tav);
	}
}

void cmd_do_lusers(User *u)
{
	send_num(u, 251, NULL, ":There are %d users and 0 invisible users on 1 servers", g_numusers);
	send_num(u, 252, NULL, "0 :operator(s) online");
	send_num(u, 254, NULL, "%d :channels formed", g_numchans);
	send_num(u, 255, NULL, ":I have %d clients and 0 servers", g_numusers);
	send_num(u, 265, NULL, ":Current Local Users: %d  Max: -1", g_numusers);
	send_num(u, 266, NULL, ":Current Global Users: %d  Max: -1", g_numusers);
}

void cmd_lusers(User *u, char *cmd, int ac, char **av)
{
	cmd_do_lusers(u);
}

void cmd_ping(User *u, char *cmd, int ac, char **av)
{
	if (ac < 1)
		send_num(u, 409, NULL, ":No origin specified");
	else
		send_cmd(u, c_sname, "PONG %s :%s", c_sname, av[0]);
}

void cmd_noop(User *u, char *cmd, int ac, char **av)
{
}

