#include "anircd.h"

int main(int ac, char **av)
{
	int		ls, maxfd, optval = 1, n;
	struct		sockaddr_in sin;

	/* Ignore SIGPIPE */
	signal(SIGPIPE, SIG_IGN);

	if (!log_init())
	{
		fprintf(stderr, "Failed to open log file; aborting.\n");
		return(-1);
	}

	if (!config_load(CONFIG_FILE))
	{
		fprintf(stderr, "Failed to open config file '%s'; aborting.\n", CONFIG_FILE);
		return(-1);
	}

	daemon(1, 0);

	ls = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (ls < 0)
	{
		perror("socket");
		return(-1);
	}

	memset(&sin, 0, sizeof(sin));
	sin.sin_family = PF_INET;
	sin.sin_port = htons(c_port);
	sin.sin_addr.s_addr = INADDR_ANY;

	if (setsockopt(ls, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval)) < 0)
	{
		perror("setsockopt");
		exit(-1);
	}

	if (bind(ls, (struct sockaddr *) &sin, sizeof(sin)) < 0)
	{
		perror("bind");
		return(-1);
	}

	if (listen(ls, 5) < 0)
	{
		perror("listen");
		return(-1);
	}

	sock_nonblock(ls);

	if (!init_SSL(c_sslkey))
		return(-1);

	for (;;)
	{
		User	*u, *un;
		struct	timeval tv;
		fd_set	rfds, wfds;

		FD_ZERO(&rfds);
		FD_ZERO(&wfds);
		FD_SET(ls, &rfds);

		tv.tv_sec = 0;
		tv.tv_usec = 500 * 1000;

		for (maxfd = ls, u = ulist; u; u = u->next)
		{
			maxfd = (u->s > maxfd) ? u->s : maxfd;
			FD_SET(u->s, &rfds);
/*			FD_SET(u->s, &wfds);*/
		}

		if (select(maxfd + 1, &rfds, &wfds, NULL, &tv) < 0)
		{
			log_error("select() returned an error: %s, exiting", strerror(errno));
			break;
		}

		if (FD_ISSET(ls, &rfds))
		{
			int	cs;

			if ((cs = accept(ls, NULL, NULL)) < 0)
				log_error("accept() failed for client: %s", strerror(errno));
			else if (!sock_nonblock(cs))
				close(cs);
			else if (!(u = user_new(cs)))
				close(cs);
			else
			{
				int	optval = 1, r;

				BIO_socket_ioctl(cs, FIONBIO, &optval);
				u->bio = BIO_new_socket(cs, BIO_NOCLOSE);

				u->ssl = SSL_new(g_SSLctx);
				SSL_set_bio(u->ssl, u->bio, u->bio);
			}
		}

		for (u = ulist; u; u = un)
		{
			/* in case struct gets deleted! */
			un = u->next;
			if (FD_ISSET(u->s, &rfds))
				while ((n = process_read(u)))
				{
					if (n == 1)
					{
						if (process_inbuf(u) < 0)
						{
							n = -1;
							break;
						}
					}
					else
						break;
				}

/*			if (FD_ISSET(u->s, &wfds)) */
			if (n >= 0)
				process_write(u);
		}
	}

	return(0);
}
